/*
 XmlFormatter.java - Stores objects to an XML file
 
 Copyright  (C) 2004 Hugues Johnson
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See
 the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/* Relies on XStream (http://xstream.codehaus.org/index.html) for XML serialization

 License for XStream follows:
 
 Copyright (c) 2003-2004, Joe Walnes
 All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:

 Redistributions of source code must retain the above copyright notice, this list of
 conditions and the following disclaimer. Redistributions in binary form must reproduce
 the above copyright notice, this list of conditions and the following disclaimer in
 the documentation and/or other materials provided with the distribution.

 Neither the name of XStream nor the names of its contributors may be used to endorse
 or promote products derived from this software without specific prior written
 permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
 SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY
 WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 DAMAGE. 
 */

package com.huguesjohnson.serialization;

import com.huguesjohnson.serialization.Formatter;
import com.huguesjohnson.serialization.Storable;
import com.huguesjohnson.serialization.StorageException;
import com.thoughtworks.xstream.XStream;
import com.thoughtworks.xstream.io.xml.DomDriver;
import java.io.File;
import java.io.FileOutputStream;
import java.io.FileReader;

/**
 * XmlFormatter - Stores objects to an XML file
 * @author Hugues Johnson
 */
public class XmlFormatter implements Formatter{
	/**
	 * Loads an object from the specified source path.
	 * @param sourcePath The path to load the object from.
	 * @return The object stored at the specified path.
	 * @throws StorageException If the load operation fails for any reason.
	 */
	public Storable loadFrom(String sourcePath) throws StorageException{
		try{
			XStream xstream=new XStream(new DomDriver());
			// load the object from the file
			Storable object=(Storable)xstream.fromXML(new FileReader(sourcePath));
			// get the absoluteDir and call the onDeserialize event
			String absoluteDir=new String(sourcePath);
			if(!absoluteDir.endsWith(File.separator)){
				int lastIndexOf=absoluteDir.lastIndexOf(File.separator);
				if((lastIndexOf>0)&&(lastIndexOf<absoluteDir.length())){
					absoluteDir=absoluteDir.substring(0,lastIndexOf+1);
				}
			}
			object.onDeserialize(this,absoluteDir);
			return(object);
		} catch(Exception x){
			x.printStackTrace();
			throw(new StorageException("Error caught in load: "+x.getMessage()));
		}
	}

	/**
	 * Saves the object to the output path.
	 * @param object The object to save.
	 * @param outputPath The path to save the object to.
	 * @throws StorageException If the save operation fails for any reason.
	 */
	public void saveTo(Storable object,String outputPath) throws StorageException{
		try{
			XStream xstream=new XStream(new DomDriver());
			String xml=xstream.toXML(object);
			FileOutputStream out=new FileOutputStream(new File(outputPath));
			out.write(xml.getBytes());
			out.flush();
			out.close();
		} catch(Exception x){
			x.printStackTrace();
			throw(new StorageException("Error caught in save: "+x.getMessage()));
		}
	}
}